| Title: | Object-Oriented Interface for Offline Change-Point Detection |
|---|---|
| Description: | A collection of efficient implementations of popular offline change-point detection algorithms, featuring a consistent, object-oriented interface for practical use. |
| Authors: | Minh Long Nguyen [aut, cre], Toby Hocking [aut], Charles Truong [aut] |
| Maintainer: | Minh Long Nguyen <[email protected]> |
| License: | CC BY 4.0 |
| Version: | 1.0.2 |
| Built: | 2026-06-09 05:56:27 UTC |
| Source: | https://github.com/edelweiss611428/rupturesrcpp |
binSeg)An R6 class implementing binary segmentation for offline change-point detection.
Binary segmentation is a classic algorithm for change-point detection that recursively splits the data at locations that minimise the cost function.
binSeg requires a R6 object of class costFunc, which can be created via costFunc$new(). Currently, the following cost functions are supported:
"L1" and "L2" for (independent) piecewise Gaussian process with constant variance
"SIGMA": for (independent) piecewise Gaussian process with varying variance
"VAR": for piecewise Gaussian vector-regressive process with constant noise variance
"LinearL2": for piecewise linear regression process with constant noise variance
See $eval() method for more details on computation of cost.
Some examples are provided below. See the GitHub README for detailed basic usage!
$new()Initialises a binSeg object.
$describe()Describes the binSeg object.
$fit()Constructs a binSeg module in C++.
$eval()Evaluates the cost of a segment.
$predict()Performs binSeg given a linear penalty value.
$plot()Plots change-point segmentation in ggplot style.
$clone()Clones the R6 object.
minSizeInteger. Minimum allowed segment length. Can be accessed or modified via $minSize.
Modifying minSize will automatically trigger $fit().
jumpInteger. Search grid step size. Can be accessed or modified via $jump.
Modifying jump will automatically trigger $fit().
costFuncR6 object of class costFunc. Search grid step size. Can be accessed or modified via $costFunc.
Modifying costFunc will automatically trigger $fit().
tsMatNumeric matrix. Input time series matrix of size . Can be accessed or modified via $tsMat.
Modifying tsMat will automatically trigger $fit().
covariatesNumeric matrix. Input time series matrix having a similar number of observations as tsMat. Can be accessed or modified via $covariates.
Modifying covariates will automatically trigger $fit().
new()
Initialises a binSeg object.
binSeg$new(minSize, jump, costFunc)
minSizeInteger. Minimum allowed segment length. Default: 1L.
jumpInteger. Search grid step size: only positions in {k, 2k, ...} are considered. Default: 1L.
costFuncA R6 object of class costFunc. Should be created via costFunc$new() to avoid error.
Default: costFunc$new("L2").
Invisibly returns NULL.
describe()
Describes a binSeg object.
binSeg$describe(printConfig = FALSE)
printConfigLogical. Whether to print object configurations. Default: FALSE.
Invisibly returns a list storing at least the following fields:
minSizeMinimum allowed segment length.
jumpSearch grid step size.
costFuncThe costFun object.
fittedWhether or not $fit() has been run.
tsMatTime series matrix.
covariatesCovariate matrix (if exists).
nNumber of observations.
pNumber of features.
fit()
Constructs a C++ module for binary segmentation.
binSeg$fit(tsMat = NULL, covariates = NULL)
tsMatNumeric matrix. A time series matrix of size whose rows are observations ordered in time.
If tsMat = NULL, the method will use the previously assigned tsMat (e.g., set via the active binding $tsMat
or from a prior $fit(tsMat)). Default: NULL.
covariatesNumeric matrix. A time series matrix having a similar number of observations as tsMat.
Required for models involving both dependent and independent variables.
If covariates = NULL and no prior covariates were set (i.e., $covariates is still NULL),
the model is force-fitted with only an intercept. Default: NULL.
This method constructs a C++ binSeg module and sets private$.fitted to TRUE,
enabling the use of $predict() and $eval(). Some precomputations are performed to allow
$predict() to run in linear time with respect to the number of data points
(see $predict() for more details).
Invisibly returns NULL.
eval()
Evaluate the cost of the segment (a,b]
binSeg$eval(a, b)
aInteger. Start index of the segment (exclusive). Must satisfy start < end.
bInteger. End index of the segment (inclusive).
The segment cost is evaluated as follows:
L1 cost function:
where is the coordinate-wise median of the segment. If , return 0.
L2 cost function:
where is the empirical mean of the segment. If , return 0.
SIGMA cost function:
where is
the empirical covariance matrix of the segment without Bessel's correction. Here, if addSmallDiag = TRUE, a small
bias epsilon is added to the diagonal of estimated covariance matrices to improve numerical stability.
By default, addSmallDiag = TRUE and epsilon = 1e-6. In case addSmallDiag = TRUE, if the computed determinant of covariance matrix is either 0 (singular)
or smaller than p*log(epsilon) - the lower bound, return (b - a)*p*log(epsilon), otherwise, output an error message.
VAR(r) cost function:
where are the estimated VAR coefficients, commonly estimated via the OLS criterion. If system is singular,
(i.e., not enough observations), or (where n is the time series length), return 0.
"LinearL2" for piecewise linear regression process with constant noise variance
where are OLS estimates on segment . If segment is shorter than the minimum number of
points needed for OLS, return 0.
The segment cost.
predict()
Performs binSeg given a linear penalty value.
binSeg$predict(pen = 0)
penNumeric. Penalty per change-point. Default: 0.
The algorithm recursively partitions a time series to detect multiple change-points. At each step, the algorithm identifies the segment that, if split, would result in the greatest reduction in total cost. This process continues until no further splits are possible (e.g., each segment is of minimal length or each breakpoint corresponds to a single data point).
Then, the algorithm selects the "optimal" set of break-points given the linear penalty threshold. Let denote the set of segment end-points with ,
where is the number of detected change-points and is the total number of data points.
and is the number of change-points. Let be the cost of segment .
The total penalised cost is then
where is a linear penalty applied per change-point. We then optimise over
to minimise the penalised cost function.
This approach allows detecting multiple change-points in a time series while controlling model complexity through the linear penalty threshold.
In our implementation, the recursive step is carried out during $fit().
Therefore, $predict() runs in linear time with respect to the number of data points.
Temporary segment end-points are saved to private$.tmpEndPoints after $predict(), enabling users to call $plot() without
specifying endpoints manually.
An integer vector of regime end-points. By design, the last element is the number of observations.
plot()
Plots change-point segmentation
binSeg$plot(
d = 1L,
endPts,
dimNames,
main,
xlab,
tsWidth = 0.25,
tsCol = "#5B9BD5",
bgCol = c("#A3C4F3", "#FBB1BD"),
bgAlpha = 0.5,
ncol = 1L
)dInteger vector. Dimensions to plot. Default: 1L.
endPtsInteger vector. End points. Default: latest temporary changepoints obtained via $predict().
dimNamesCharacter vector. Feature names matching length of d. Defaults to "X1", "X2", ....
mainCharacter. Main title. Defaults to "binSeg: d = ...".
xlabCharacter. X-axis label. Default: "Time".
tsWidthNumeric. Line width for time series and segments. Default: 0.25.
tsColCharacter. Time series color. Default: "#5B9BD5".
bgColCharacter vector. Segment colors, recycled to length of endPts. Default: c("#A3C4F3", "#FBB1BD").
bgAlphaNumeric. Background transparency. Default: 0.5.
ncolInteger. Number of columns in facet layout. Default: 1L.
Plots change-point segmentation results. Based on ggplot2. Multiple plots can easily be
horizontally and vertically stacked using patchwork's operators / and |, respectively.
An object of classes gg and ggplot.
clone()
The objects of this class are cloneable with this method.
binSeg$clone(deep = FALSE)
deepWhether to make a deep clone.
Minh Long Nguyen [email protected]
Toby Dylan Hocking [email protected]
Charles Truong [email protected]
Truong, C., Oudre, L., & Vayatis, N. (2020). Selective review of offline change point detection methods. Signal Processing, 167, 107299.
Hocking, T. D. (2024). Finite Sample Complexity Analysis of Binary Segmentation. arXiv preprint arXiv:2410.08654.
## L2 example set.seed(1121) signals = as.matrix(c(rnorm(100,0,1), rnorm(100,5,1))) # Default L2 cost function binSegObj = binSeg$new(minSize = 1L, jump = 1L) binSegObj$fit(signals) binSegObj$predict(pen = 100) binSegObj$plot() ## SIGMA example set.seed(111) signals = as.matrix(c(rnorm(100,-5,1), rnorm(100,-5,10), rnorm(100,-5,1))) # L2 cost function binSegObj = binSeg$new(minSize = 1L, jump = 1L) binSegObj$fit(signals) # We choose pen = 50. binSegObj$predict(pen = 50) binSegObj$plot() # The standard L2 cost function is not suitable. # Use the SIGMA cost function. binSegObj$costFunc = costFunc$new(costFunc = "SIGMA") binSegObj$predict(pen = 50) binSegObj$plot()## L2 example set.seed(1121) signals = as.matrix(c(rnorm(100,0,1), rnorm(100,5,1))) # Default L2 cost function binSegObj = binSeg$new(minSize = 1L, jump = 1L) binSegObj$fit(signals) binSegObj$predict(pen = 100) binSegObj$plot() ## SIGMA example set.seed(111) signals = as.matrix(c(rnorm(100,-5,1), rnorm(100,-5,10), rnorm(100,-5,1))) # L2 cost function binSegObj = binSeg$new(minSize = 1L, jump = 1L) binSegObj$fit(signals) # We choose pen = 50. binSegObj$predict(pen = 50) binSegObj$plot() # The standard L2 cost function is not suitable. # Use the SIGMA cost function. binSegObj$costFunc = costFunc$new(costFunc = "SIGMA") binSegObj$predict(pen = 50) binSegObj$plot()
costFunc classAn R6 class specifying a cost function
Creates an instance of costFunc R6 class, used in initialisation of change-point detection modules. Currently
supports the following cost functions:
L1 cost function:
where is the coordinate-wise median of the segment. If , return 0.
L2 cost function:
where is the empirical mean of the segment. If , return 0.
SIGMA cost function:
where is
the empirical covariance matrix of the segment without Bessel's correction. Here, if addSmallDiag = TRUE, a small
bias epsilon is added to the diagonal of estimated covariance matrices to improve numerical stability.
By default, addSmallDiag = TRUE and epsilon = 1e-6. In case addSmallDiag = TRUE, if the computed determinant of covariance matrix is either 0 (singular)
or smaller than p*log(epsilon) - the lower bound, return (b - a)*p*log(epsilon), otherwise, output an error message.
VAR(r) cost function:
where are the estimated VAR coefficients, commonly estimated via the OLS criterion. If system is singular,
(i.e., not enough observations), or (where n is the time series length), return 0.
"LinearL2" for piecewise linear regression process with constant noise variance
where are OLS estimates on segment . If segment is shorter than the minimum number of
points needed for OLS, return 0.
If active binding $costFunc is modified (via assignment operator), the default parameters will be used.
$new()Initialises a costFunc object.
$pass()Describes the costFunc object.
$clone()Clones the costFunc object.
costFuncCharacter. Cost function. Can be accessed or modified via $costFunc. If costFunc is modified
and required parameters are missing, the default parameters are used.
pVARInteger. Vector autoregressive order. Can be accessed or modified via $pVAR.
addSmallDiagLogical. Whether to add a bias value to the diagonal of estimated covariance matrices to stabilise matrix operations. Can be accessed or modified via $addSmallDiag.
epsilonDouble. A bias value added to the diagonal of estimated covariance matrices to stabilise matrix operations. Can be accessed or modified via $epsilon.
interceptLogical. Whether to include the intercept in regression problems. Can be accessed or modified via $intercept.
new()
Initialises a costFunc object.
costFunc$new(costFunc, ...)
costFuncCharacter. Cost function. Supported values include "L2", "VAR",
and "SIGMA". Default: L2.
...Optional named parameters required by specific cost functions.
If any required parameters are missing or null, default values will be used.
For "L1" and "L2", there is no extra parameter.
For "SIGMA", supported parameters are:
addSmallDiagLogical. If TRUE, add a small value to the diagonal of estimated covariance matrices
to stabilise matrix operations. Default: TRUE.
epsilonDouble. If addSmallDiag = TRUE, a small positive value added to the diagonal of estimated covariance matrices to stabilise
matrix operations. Default: 1e-6.
For "VAR", pVAR is required:
pVARInteger. Vector autoregressive order. Must be a positive integer. Default: 1L.
For "LinearL2", intercept is required:
interceptLogical. Whether to include the intercept in regression problems. Default: TRUE.
pass()
Returns a list of configuration parameters to initialise detection modules.
costFunc$pass()
clone()
The objects of this class are cloneable with this method.
costFunc$clone(deep = FALSE)
deepWhether to make a deep clone.
Minh Long Nguyen [email protected]
## L2 costFunc (default) costFuncObj = costFunc$new() costFuncObj$pass() ## SIGMA costFunc costFuncObj = costFunc$new(costFunc = "SIGMA") costFuncObj$pass() # Modify active bindings costFuncObj$epsilon = 10^-5 costFuncObj$pass() costFuncObj$costFunc = "VAR" costFuncObj$pass()## L2 costFunc (default) costFuncObj = costFunc$new() costFuncObj$pass() ## SIGMA costFunc costFuncObj = costFunc$new(costFunc = "SIGMA") costFuncObj$pass() # Modify active bindings costFuncObj$epsilon = 10^-5 costFuncObj$pass() costFuncObj$costFunc = "VAR" costFuncObj$pass()
PELT)An R6 class implementing the PELT algorithm for offline change-point detection.
PELT (Pruned Exact Linear Time) is an efficient algorithm for change point detection that prunes the search space to achieve optimal segmentation in linear time under certain conditions.
PELT requires a R6 object of class costFunc, which can be created via costFunc$new(). Currently, the following cost functions are supported:
"L1" and "L2" for (independent) piecewise Gaussian process with constant variance
"SIGMA": for (independent) piecewise Gaussian process with varying variance
"VAR": for piecewise Gaussian vector-regressive process with constant noise variance
"LinearL2": for piecewise linear regression process with constant noise variance
See $eval() method for more details on computation of cost.
Some examples are provided below. See the GitHub README for detailed basic usage!
$new()Initialises a PELT object.
$describe()Describes the PELT object.
$fit()Constructs a PELT module in C++.
$eval()Evaluates the cost of a segment.
$predict()Performs PELT given a linear penalty value.
$plot()Plots change-point segmentation in ggplot style.
$clone()Clones the R6 object.
minSizeInteger. Minimum allowed segment length. Can be accessed or modified via $minSize.
Modifying minSize will automatically trigger $fit().
jumpInteger. Search grid step size. Can be accessed or modified via $jump.
Modifying jump will automatically trigger $fit().
costFuncR6 object of class costFunc. Search grid step size. Can be accessed or modified via $costFunc.
Modifying costFunc will automatically trigger $fit().
tsMatNumeric matrix. Input time series matrix of size . Can be accessed or modified via $tsMat.
Modifying tsMat will automatically trigger $fit().
covariatesNumeric matrix. Input time series matrix having a similar number of observations as tsMat. Can be accessed or modified via $covariates.
Modifying covariates will automatically trigger $fit().
new()
Initialises a PELT object.
PELT$new(minSize, jump, costFunc)
minSizeInteger. Minimum allowed segment length. Default: 1L.
jumpInteger. Search grid step size: only positions in {k, 2k, ...} are considered. Default: 1L.
costFuncA R6 object of class costFunc. Should be created via costFunc$new() to avoid error.
Default: costFunc$new("L2").
Invisibly returns NULL.
describe()
Describes a PELT object.
PELT$describe(printConfig = FALSE)
printConfigLogical. Whether to print object configurations. Default: FALSE.
Invisibly returns a list storing at least the following fields:
minSizeMinimum allowed segment length.
jumpSearch grid step size.
costFuncThe costFun object.
fittedWhether or not $fit() has been run.
tsMatTime series matrix.
covariatesCovariate matrix (if exists).
nNumber of observations.
pNumber of features.
fit()
Constructs a C++ module for PELT.
PELT$fit(tsMat = NULL, covariates = NULL)
tsMatNumeric matrix. A time series matrix of size whose rows are observations ordered in time.
If tsMat = NULL, the method will use the previously assigned tsMat (e.g., set via the active binding $tsMat
or from a prior $fit(tsMat)). Default: NULL.
covariatesNumeric matrix. A time series matrix having a similar number of observations as tsMat.
Required for models involving both dependent and independent variables.
If covariates = NULL and no prior covariates were set (i.e., $covariates is still NULL),
the model is force-fitted with only an intercept. Default: NULL.
This method constructs a C++ PELT module and sets private$.fitted to TRUE, enabling the use of $predict() and $eval().
Invisibly returns NULL.
eval()
Evaluate the cost of the segment (a,b]
PELT$eval(a, b)
aInteger. Start index of the segment (exclusive). Must satisfy start < end.
bInteger. End index of the segment (inclusive).
The segment cost is evaluated as follows:
L1 cost function:
where is the coordinate-wise median of the segment. If , return 0.
L2 cost function:
where is the empirical mean of the segment. If , return 0.
SIGMA cost function:
where is
the empirical covariance matrix of the segment without Bessel's correction. Here, if addSmallDiag = TRUE, a small
bias epsilon is added to the diagonal of estimated covariance matrices to improve numerical stability.
By default, addSmallDiag = TRUE and epsilon = 1e-6. In case addSmallDiag = TRUE, if the computed determinant of covariance matrix is either 0 (singular)
or smaller than p*log(epsilon) - the lower bound, return (b - a)*p*log(epsilon), otherwise, output an error message.
VAR(r) cost function:
where are the estimated VAR coefficients, commonly estimated via the OLS criterion. If system is singular,
(i.e., not enough observations), or (where n is the time series length), return 0.
"LinearL2" for piecewise linear regression process with constant noise variance
where are OLS estimates on segment . If segment is shorter than the minimum number of
points needed for OLS, return 0.
The segment cost.
predict()
Performs PELT given a linear penalty value.
PELT$predict(pen = 0)
penNumeric. Penalty per change-point. Default: 0.
The PELT algorithm detects multiple change-points by finding the set of break-points that globally minimises
a penalised cost function. PELT uses dynamic programming combined with a pruning rule to reduce the number of candidate change-points, achieving efficient computation.
Let denote the set of segment end-points with ,
where is the number of detected change-points and is the total number of data points.
Let be the cost of segment .
The total penalised cost is
where is a linear penalty applied per change-point. PELT finds the set of endpoints that minimises this cost exactly.
The pruning step eliminates candidate change-points that cannot lead to an optimal solution,
allowing PELT to run in linear time with respect to the number of data points.
Temporary segment end-points are saved to private$.tmpEndPoints after $predict(), enabling users to call $plot() without
specifying endpoints manually.
An integer vector of regime end-points. By design, the last element is the number of observations.
plot()
Plots change-point segmentation
PELT$plot(
d = 1L,
endPts,
dimNames,
main,
xlab,
tsWidth = 0.25,
tsCol = "#5B9BD5",
bgCol = c("#A3C4F3", "#FBB1BD"),
bgAlpha = 0.5,
ncol = 1L
)dInteger vector. Dimensions to plot. Default: 1L.
endPtsInteger vector. End points. Default: latest temporary changepoints obtained via $predict().
dimNamesCharacter vector. Feature names matching length of d. Defaults to "X1", "X2", ....
mainCharacter. Main title. Defaults to "PELT: d = ...".
xlabCharacter. X-axis label. Default: "Time".
tsWidthNumeric. Line width for time series and segments. Default: 0.25.
tsColCharacter. Time series color. Default: "#5B9BD5".
bgColCharacter vector. Segment colors, recycled to length of endPts. Default: c("#A3C4F3", "#FBB1BD").
bgAlphaNumeric. Background transparency. Default: 0.5.
ncolInteger. Number of columns in facet layout. Default: 1L.
Plots change-point segmentation results. Based on ggplot2. Multiple plots can easily be
horizontally and vertically stacked using patchwork's operators / and |, respectively.
An object of classes gg and ggplot.
clone()
The objects of this class are cloneable with this method.
PELT$clone(deep = FALSE)
deepWhether to make a deep clone.
Minh Long Nguyen [email protected]
Toby Dylan Hocking [email protected]
Charles Truong [email protected]
Truong, C., Oudre, L., & Vayatis, N. (2020). Selective review of offline change point detection methods. Signal Processing, 167, 107299.
Killick, R., Fearnhead, P., & Eckley, I. A. (2012). Optimal detection of change points with a linear computational cost. Journal of the American Statistical Association, 107(500), 1590-1598.
## L2 example set.seed(1121) signals = as.matrix(c(rnorm(100,0,1), rnorm(100,5,1))) # Default L2 cost function PELTObj = PELT$new(minSize = 1L, jump = 1L) PELTObj$fit(signals) PELTObj$predict(pen = 100) PELTObj$plot() ## SIGMA example set.seed(111) signals = as.matrix(c(rnorm(100,-5,1), rnorm(100,-5,10), rnorm(100,-5,1))) # L2 cost function PELTObj = PELT$new(minSize = 1L, jump = 1L) PELTObj$fit(signals) # We choose pen = 50. PELTObj$predict(pen = 50) PELTObj$plot() # The standard L2 cost function is not suitable. # Use the SIGMA cost function. PELTObj$costFunc = costFunc$new(costFunc = "SIGMA") PELTObj$predict(pen = 50) PELTObj$plot()## L2 example set.seed(1121) signals = as.matrix(c(rnorm(100,0,1), rnorm(100,5,1))) # Default L2 cost function PELTObj = PELT$new(minSize = 1L, jump = 1L) PELTObj$fit(signals) PELTObj$predict(pen = 100) PELTObj$plot() ## SIGMA example set.seed(111) signals = as.matrix(c(rnorm(100,-5,1), rnorm(100,-5,10), rnorm(100,-5,1))) # L2 cost function PELTObj = PELT$new(minSize = 1L, jump = 1L) PELTObj$fit(signals) # We choose pen = 50. PELTObj$predict(pen = 50) PELTObj$plot() # The standard L2 cost function is not suitable. # Use the SIGMA cost function. PELTObj$costFunc = costFunc$new(costFunc = "SIGMA") PELTObj$predict(pen = 50) PELTObj$plot()
Window)An R6 class implementing slicing window for offline change-point detection.
Slicing window is a scalable, linear-time change-point detection algorithm that selects breakpoints based on local gains computed over sliding windows.
Currently supports the following cost functions:
"L1" and "L2" for (independent) piecewise Gaussian process with constant variance
"SIGMA": for (independent) piecewise Gaussian process with varying variance
"VAR": for piecewise Gaussian vector-regressive process with constant noise variance
"LinearL2": for piecewise linear regression process with constant noise variance
Window requires a R6 object of class costFunc, which can be created via costFunc$new(). Currently, the following cost functions are supported:
"L1" and "L2" for (independent) piecewise Gaussian process with constant variance
"SIGMA": for (independent) piecewise Gaussian process with varying variance
"VAR": for piecewise Gaussian vector-regressive process with constant noise variance
"LinearL2": for piecewise linear regression process with constant noise variance
See $eval() method for more details on computation of cost.
Some examples are provided below. See the GitHub README for detailed basic usage!
$new()Initialises a Window object.
$describe()Describes the Window object.
$fit()Constructs a Window module in C++.
$eval()Evaluates the cost of a segment.
$predict()Performs Window given a linear penalty value.
$plot()Plots change-point segmentation in ggplot style.
$clone()Clones the R6 object.
minSizeInteger. Minimum allowed segment length. Can be accessed or modified via $minSize.
Modifying minSize will automatically trigger $fit().
radiusInteger. Window radius. Can be accessed or modified via $radius.
Modifying radius will automatically trigger $fit().
jumpInteger. Search grid step size. Can be accessed or modified via $jump.
Modifying jump will automatically trigger $fit().
costFuncR6 object of class costFunc. Search grid step size. Can be accessed or modified via $costFunc.
Modifying costFunc will automatically trigger $fit().
tsMatNumeric matrix. Input time series matrix of size . Can be accessed or modified via $tsMat.
Modifying tsMat will automatically trigger $fit().
covariatesNumeric matrix. Input time series matrix having a similar number of observations as tsMat. Can be accessed or modified via $covariates.
Modifying covariates will automatically trigger $fit().
new()
Initialises a Window object.
Window$new(minSize, jump, radius, costFunc)
minSizeInteger. Minimum allowed segment length. Default: 1L.
jumpInteger. Search grid step size: only positions in {k, 2k, ...} are considered. Default: 1L.
radiusInteger. Radius of each sliding window. Default: 1L.
costFuncA R6 object of class costFunc. Should be created via costFunc$new() to avoid error.
Default: costFunc$new("L2").
Invisibly returns NULL.
describe()
Describes a Window object.
Window$describe(printConfig = FALSE)
printConfigLogical. Whether to print object configurations. Default: FALSE.
Invisibly returns a list storing at least the following fields:
minSizeMinimum allowed segment length.
jumpSearch grid step size.
radiusRadius of each sliding window.
costFuncThe costFun object.
fittedWhether or not $fit() has been run.
tsMatTime series matrix.
covariatesCovariate matrix (if exists).
nNumber of observations.
pNumber of features.
fit()
Constructs a C++ module for Window.
Window$fit(tsMat = NULL, covariates = NULL)
tsMatNumeric matrix. A time series matrix of size whose rows are observations ordered in time.
If tsMat = NULL, the method will use the previously assigned tsMat (e.g., set via the active binding $tsMat
or from a prior $fit(tsMat)). Default: NULL.
covariatesNumeric matrix. A time series matrix having a similar number of observations as tsMat.
Required for models involving both dependent and independent variables.
If covariates = NULL and no prior covariates were set (i.e., $covariates is still NULL),
the model is force-fitted with only an intercept. Default: NULL..
This method constructs a C++ Window module and sets private$.fitted to TRUE,
enabling the use of $predict() and $eval(). Some precomputations are performed to allow
$predict() to run in linear time with respect to the number of local change-points
(see $predict() for more details).
Invisibly returns NULL.
eval()
Evaluate the cost of the segment (a,b]
Window$eval(a, b)
aInteger. Start index of the segment (exclusive). Must satisfy start < end.
bInteger. End index of the segment (inclusive).
The segment cost is evaluated as follows:
L1 cost function:
where is the coordinate-wise median of the segment. If , return 0.
L2 cost function:
where is the empirical mean of the segment. If , return 0.
SIGMA cost function:
where is
the empirical covariance matrix of the segment without Bessel's correction. Here, if addSmallDiag = TRUE, a small
bias epsilon is added to the diagonal of estimated covariance matrices to improve numerical stability.
By default, addSmallDiag = TRUE and epsilon = 1e-6. In case addSmallDiag = TRUE, if the computed determinant of covariance matrix is either 0 (singular)
or smaller than p*log(epsilon) - the lower bound, return (b - a)*p*log(epsilon), otherwise, output an error message.
VAR(r) cost function:
where are the estimated VAR coefficients, commonly estimated via the OLS criterion. If system is singular,
(i.e., not enough observations), or (where n is the time series length), return 0.
"LinearL2" for piecewise linear regression process with constant noise variance
where are OLS estimates on segment . If segment is shorter than the minimum number of
points needed for OLS, return 0.
The segment cost.
predict()
Performs Window given a linear penalty value.
Window$predict(pen = 0)
penNumeric. Penalty per change-point. Default: 0.
The algorithm scans the data with a fixed-size window to detect candidate local change-points (lcps)
if the gains of its neighbors to the left and right are all smaller than its gain,
where is defined as
After candidate local change-points and computing the local gains, the algorithm selects the "optimal" set of break-points given the linear penalty threshold.
Let denote the local gain for candidate change-point , for . The local gains are ordered such that
. Note that it is possible that no local change-points are detected,
for example if the window size is too large.
The total cost for the selected change-points is then calculated as
where is a linear penalty applied per change-point. We then optimise over
to minimise the penalised cost function.
This approach allows detecting multiple change-points in a time series while controlling model complexity through the linear penalty threshold.
In our implementation, scanning the data to detect candidate local change-points and computing their corresponding local gains is already performed in $fit().
Therefore, $predict() runs in linear time with respect to the number of local change-points.
Temporary segment end-points are saved to private$.tmpEndPoints after $predict(), enabling users to call $plot() without
specifying endpoints manually.
An integer vector of regime end-points. By design, the last element is the number of observations.
plot()
Plots change-point segmentation
Window$plot(
d = 1L,
endPts,
dimNames,
main,
xlab,
tsWidth = 0.25,
tsCol = "#5B9BD5",
bgCol = c("#A3C4F3", "#FBB1BD"),
bgAlpha = 0.5,
ncol = 1L
)dInteger vector. Dimensions to plot. Default: 1L.
endPtsInteger vector. End points. Default: latest temporary changepoints obtained via $predict().
dimNamesCharacter vector. Feature names matching length of d. Defaults to "X1", "X2", ....
mainCharacter. Main title. Defaults to "Window: d = ...".
xlabCharacter. X-axis label. Default: "Time".
tsWidthNumeric. Line width for time series and segments. Default: 0.25.
tsColCharacter. Time series color. Default: "#5B9BD5".
bgColCharacter vector. Segment colors, recycled to length of endPts. Default: c("#A3C4F3", "#FBB1BD").
bgAlphaNumeric. Background transparency. Default: 0.5.
ncolInteger. Number of columns in facet layout. Default: 1L.
Plots change-point segmentation results. Based on ggplot2. Multiple plots can easily be
horizontally and vertically stacked using patchwork's operators / and |, respectively.
An object of classes gg and ggplot.
clone()
The objects of this class are cloneable with this method.
Window$clone(deep = FALSE)
deepWhether to make a deep clone.
Minh Long Nguyen [email protected]
Toby Dylan Hocking [email protected]
Charles Truong [email protected]
Truong, C., Oudre, L., & Vayatis, N. (2020). Selective review of offline change point detection methods. Signal Processing, 167, 107299.
## L2 example set.seed(1121) signals = as.matrix(c(rnorm(100,0,1), rnorm(100,5,1))) # Default L2 cost function WindowObj = Window$new(minSize = 1L, jump = 1L) WindowObj$fit(signals) WindowObj$predict(pen = 100) WindowObj$plot() ## SIGMA example set.seed(111) signals = as.matrix(c(rnorm(100,-5,1), rnorm(100,-5,10), rnorm(100,-5,1))) # L2 cost function WindowObj = Window$new(minSize = 1L, jump = 1L) WindowObj$fit(signals) # We choose pen = 50. WindowObj$predict(pen = 50) WindowObj$plot() # The standard L2 cost function is not suitable. # Use the SIGMA cost function. WindowObj$costFunc = costFunc$new(costFunc = "SIGMA") WindowObj$predict(pen = 50) WindowObj$plot()## L2 example set.seed(1121) signals = as.matrix(c(rnorm(100,0,1), rnorm(100,5,1))) # Default L2 cost function WindowObj = Window$new(minSize = 1L, jump = 1L) WindowObj$fit(signals) WindowObj$predict(pen = 100) WindowObj$plot() ## SIGMA example set.seed(111) signals = as.matrix(c(rnorm(100,-5,1), rnorm(100,-5,10), rnorm(100,-5,1))) # L2 cost function WindowObj = Window$new(minSize = 1L, jump = 1L) WindowObj$fit(signals) # We choose pen = 50. WindowObj$predict(pen = 50) WindowObj$plot() # The standard L2 cost function is not suitable. # Use the SIGMA cost function. WindowObj$costFunc = costFunc$new(costFunc = "SIGMA") WindowObj$predict(pen = 50) WindowObj$plot()